@@ -1,31 +1,22 @@
-# copyright 2003-2014 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
-# contact http://www.logilab.fr/ -- mailto:contact@logilab.fr
-#
-# This file is part of astroid.
-#
-# astroid is free software: you can redistribute it and/or modify it
-# under the terms of the GNU Lesser General Public License as published by the
-# Free Software Foundation, either version 2.1 of the License, or (at your
-# option) any later version.
-#
-# astroid is distributed in the hope that it will be useful, but
-# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License
-# for more details.
-#
-# You should have received a copy of the GNU Lesser General Public License along
-# with astroid. If not, see <http://www.gnu.org/licenses/>.
+# Copyright (c) 2006-2011, 2013-2014 LOGILAB S.A. (Paris, FRANCE) <contact@logilab.fr>
+# Copyright (c) 2014-2015 Google, Inc.
+# Copyright (c) 2014-2016 Claudiu Popa <pcmanticore@gmail.com>
+# Copyright (c) 2015-2016 Cara Vinson <ceridwenv@gmail.com>
+
+# Licensed under the LGPL: https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html
+# For details: https://github.com/PyCQA/astroid/blob/master/COPYING.LESSER
+
 """The AstroidBuilder makes astroid from living object and / or from _ast
 
 The builder is not thread safe and can't be used to parse different sources
 at the same time.
 """
-from __future__ import with_statement
 
-import _ast
+import re
 import os
 import sys
 import textwrap
+import _ast
 
 from astroid import bases
 from astroid import exceptions
@@ -33,32 +24,34 @@ from astroid import manager
 from astroid import modutils
 from astroid import raw_building
 from astroid import rebuilder
+from astroid import nodes
 from astroid import util
 
+# The name of the transient function that is used to
+# wrap expressions to be extracted when calling
+# extract_node.
+_TRANSIENT_FUNCTION = '__'
+
+# The comment used to select a statement to be extracted
+# when calling extract_node.
+_STATEMENT_SELECTOR = '#@'
+
 
 def _parse(string):
     return compile(string, "<string>", 'exec', _ast.PyCF_ONLY_AST)
 
 
 if sys.version_info >= (3, 0):
-    # pylint: disable=no-name-in-module; We don't understand flows yet.
     from tokenize import detect_encoding
 
     def open_source_file(filename):
         with open(filename, 'rb') as byte_stream:
             encoding = detect_encoding(byte_stream.readline)[0]
         stream = open(filename, 'r', newline=None, encoding=encoding)
-        try:
-            data = stream.read()
-        except UnicodeError:  # wrong encoding
-            # detect_encoding returns utf-8 if no encoding specified
-            msg = 'Wrong (%s) or no encoding specified' % encoding
-            raise exceptions.AstroidBuildingException(msg)
+        data = stream.read()
         return stream, encoding, data
 
 else:
-    import re
-
     _ENCODING_RGX = re.compile(r"\s*#+.*coding[:=]\s*([-\w.]+)")
 
     def _guess_encoding(string):
@@ -83,6 +76,17 @@ else:
 MANAGER = manager.AstroidManager()
 
 
+def _can_assign_attr(node, attrname):
+    try:
+        slots = node.slots()
+    except NotImplementedError:
+        pass
+    else:
+        if slots and attrname not in set(slot.value for slot in slots):
+            return False
+    return True
+
+
 class AstroidBuilder(raw_building.InspectBuilder):
     """Class for building an astroid tree from source code or from a live module.
 
@@ -92,7 +96,7 @@ class AstroidBuilder(raw_building.InspectBuilder):
     applied after the tree was built from source or from a live object,
     by default being True.
     """
-
+    # pylint: disable=redefined-outer-name
     def __init__(self, manager=None, apply_transforms=True):
         super(AstroidBuilder, self).__init__()
         self._manager = manager or MANAGER
@@ -124,12 +128,18 @@ class AstroidBuilder(raw_building.InspectBuilder):
         try:
             stream, encoding, data = open_source_file(path)
         except IOError as exc:
-            msg = 'Unable to load file %r (%s)' % (path, exc)
-            raise exceptions.AstroidBuildingException(msg)
-        except SyntaxError as exc:  # py3k encoding specification error
-            raise exceptions.AstroidBuildingException(exc)
-        except LookupError as exc:  # unknown encoding
-            raise exceptions.AstroidBuildingException(exc)
+            util.reraise(exceptions.AstroidBuildingError(
+                'Unable to load file {path}:\n{error}',
+                modname=modname, path=path, error=exc))
+        except (SyntaxError, LookupError) as exc:
+            util.reraise(exceptions.AstroidSyntaxError(
+                'Python 3 encoding specification error or unknown encoding:\n'
+                '{error}', modname=modname, path=path, error=exc))
+        except UnicodeError:  # wrong encoding
+            # detect_encoding returns utf-8 if no encoding specified
+            util.reraise(exceptions.AstroidBuildingError(
+                'Wrong or no encoding specified for {filename}.',
+                filename=path))
         with stream:
             # get module name if necessary
             if modname is None:
@@ -144,7 +154,7 @@ class AstroidBuilder(raw_building.InspectBuilder):
     def string_build(self, data, modname='', path=None):
         """Build astroid from source code string."""
         module = self._data_build(data, modname, path)
-        module.source_code = data.encode('utf-8')
+        module.file_bytes = data.encode('utf-8')
         return self._post_build(module, 'utf-8')
 
     def _post_build(self, module, encoding):
@@ -155,7 +165,7 @@ class AstroidBuilder(raw_building.InspectBuilder):
         for from_node in module._import_from_nodes:
             if from_node.modname == '__future__':
                 for symbol, _ in from_node.names:
-                    module._future_imports.add(symbol)
+                    module.future_imports.add(symbol)
             self.add_from_names_to_locals(from_node)
         # handle delayed assattr nodes
         for delayed in module._delayed_assattr:
@@ -171,7 +181,9 @@ class AstroidBuilder(raw_building.InspectBuilder):
         try:
             node = _parse(data + '\n')
         except (TypeError, ValueError, SyntaxError) as exc:
-            raise exceptions.AstroidBuildingException(exc)
+            util.reraise(exceptions.AstroidSyntaxError(
+                'Parsing Python code failed:\n{error}',
+                source=data, modname=modname, path=path, error=exc))
         if path is not None:
             node_file = os.path.abspath(path)
         else:
@@ -180,7 +192,7 @@ class AstroidBuilder(raw_building.InspectBuilder):
             modname = modname[:-9]
             package = True
         else:
-            package = path and path.find('__init__.py') > -1 or False
+            package = path.find('__init__.py') > -1 if path else False
         builder = rebuilder.TreeRebuilder(self._manager)
         module = builder.visit_module(node, modname, node_file, package)
         module._import_from_nodes = builder._import_from_nodes
@@ -200,14 +212,14 @@ class AstroidBuilder(raw_building.InspectBuilder):
             if name == '*':
                 try:
                     imported = node.do_import_module()
-                except exceptions.InferenceError:
+                except exceptions.AstroidBuildingError:
                     continue
-                for name in imported._public_names():
+                for name in imported.public_names():
                     node.parent.set_local(name, node)
-                    sort_locals(node.parent.scope()._locals[name])
+                    sort_locals(node.parent.scope().locals[name])
             else:
                 node.parent.set_local(asname or name, node)
-                sort_locals(node.parent.scope()._locals[asname or name])
+                sort_locals(node.parent.scope().locals[asname or name])
 
     def delayed_assattr(self, node):
         """Visit a AssAttr node
@@ -217,20 +229,22 @@ class AstroidBuilder(raw_building.InspectBuilder):
         try:
             frame = node.frame()
             for inferred in node.expr.infer():
-                if inferred is util.YES:
+                if inferred is util.Uninferable:
                     continue
                 try:
                     if inferred.__class__ is bases.Instance:
                         inferred = inferred._proxied
-                        iattrs = inferred._instance_attrs
+                        iattrs = inferred.instance_attrs
+                        if not _can_assign_attr(inferred, node.attrname):
+                            continue
                     elif isinstance(inferred, bases.Instance):
                         # Const, Tuple, ... we may be wrong, may be not, but
                         # anyway we don't want to pollute builtin's namespace
                         continue
                     elif inferred.is_function:
-                        iattrs = inferred._instance_attrs
+                        iattrs = inferred.instance_attrs
                     else:
-                        iattrs = inferred._locals
+                        iattrs = inferred.locals
                 except AttributeError:
                     # XXX log error
                     continue
@@ -239,7 +253,7 @@ class AstroidBuilder(raw_building.InspectBuilder):
                     continue
                 # get assign in __init__ first XXX useful ?
                 if (frame.name == '__init__' and values and
-                        not values[0].frame().name == '__init__'):
+                        values[0].frame().name != '__init__'):
                     values.insert(0, node)
                 else:
                     values.append(node)
@@ -247,6 +261,10 @@ class AstroidBuilder(raw_building.InspectBuilder):
             pass
 
 
+def build_namespace_package_module(name, path):
+    return nodes.Module(name, doc='', path=path, package=True)
+
+
 def parse(code, module_name='', path=None, apply_transforms=True):
     """Parses a source string in order to obtain an astroid AST from it
 
@@ -261,3 +279,159 @@ def parse(code, module_name='', path=None, apply_transforms=True):
     builder = AstroidBuilder(manager=MANAGER,
                              apply_transforms=apply_transforms)
     return builder.string_build(code, modname=module_name, path=path)
+
+
+def _extract_expressions(node):
+    """Find expressions in a call to _TRANSIENT_FUNCTION and extract them.
+
+    The function walks the AST recursively to search for expressions that
+    are wrapped into a call to _TRANSIENT_FUNCTION. If it finds such an
+    expression, it completely removes the function call node from the tree,
+    replacing it by the wrapped expression inside the parent.
+
+    :param node: An astroid node.
+    :type node:  astroid.bases.NodeNG
+    :yields: The sequence of wrapped expressions on the modified tree
+    expression can be found.
+    """
+    if (isinstance(node, nodes.Call)
+            and isinstance(node.func, nodes.Name)
+            and node.func.name == _TRANSIENT_FUNCTION):
+        real_expr = node.args[0]
+        real_expr.parent = node.parent
+        # Search for node in all _astng_fields (the fields checked when
+        # get_children is called) of its parent. Some of those fields may
+        # be lists or tuples, in which case the elements need to be checked.
+        # When we find it, replace it by real_expr, so that the AST looks
+        # like no call to _TRANSIENT_FUNCTION ever took place.
+        for name in node.parent._astroid_fields:
+            child = getattr(node.parent, name)
+            if isinstance(child, (list, tuple)):
+                for idx, compound_child in enumerate(child):
+                    if compound_child is node:
+                        child[idx] = real_expr
+            elif child is node:
+                setattr(node.parent, name, real_expr)
+        yield real_expr
+    else:
+        for child in node.get_children():
+            for result in _extract_expressions(child):
+                yield result
+
+
+def _find_statement_by_line(node, line):
+    """Extracts the statement on a specific line from an AST.
+
+    If the line number of node matches line, it will be returned;
+    otherwise its children are iterated and the function is called
+    recursively.
+
+    :param node: An astroid node.
+    :type node: astroid.bases.NodeNG
+    :param line: The line number of the statement to extract.
+    :type line: int
+    :returns: The statement on the line, or None if no statement for the line
+      can be found.
+    :rtype:  astroid.bases.NodeNG or None
+    """
+    if isinstance(node, (nodes.ClassDef, nodes.FunctionDef)):
+        # This is an inaccuracy in the AST: the nodes that can be
+        # decorated do not carry explicit information on which line
+        # the actual definition (class/def), but .fromline seems to
+        # be close enough.
+        node_line = node.fromlineno
+    else:
+        node_line = node.lineno
+
+    if node_line == line:
+        return node
+
+    for child in node.get_children():
+        result = _find_statement_by_line(child, line)
+        if result:
+            return result
+
+    return None
+
+
+def extract_node(code, module_name=''):
+    """Parses some Python code as a module and extracts a designated AST node.
+
+    Statements:
+     To extract one or more statement nodes, append #@ to the end of the line
+
+     Examples:
+       >>> def x():
+       >>>   def y():
+       >>>     return 1 #@
+
+       The return statement will be extracted.
+
+       >>> class X(object):
+       >>>   def meth(self): #@
+       >>>     pass
+
+      The function object 'meth' will be extracted.
+
+    Expressions:
+     To extract arbitrary expressions, surround them with the fake
+     function call __(...). After parsing, the surrounded expression
+     will be returned and the whole AST (accessible via the returned
+     node's parent attribute) will look like the function call was
+     never there in the first place.
+
+     Examples:
+       >>> a = __(1)
+
+       The const node will be extracted.
+
+       >>> def x(d=__(foo.bar)): pass
+
+       The node containing the default argument will be extracted.
+
+       >>> def foo(a, b):
+       >>>   return 0 < __(len(a)) < b
+
+       The node containing the function call 'len' will be extracted.
+
+    If no statements or expressions are selected, the last toplevel
+    statement will be returned.
+
+    If the selected statement is a discard statement, (i.e. an expression
+    turned into a statement), the wrapped expression is returned instead.
+
+    For convenience, singleton lists are unpacked.
+
+    :param str code: A piece of Python code that is parsed as
+    a module. Will be passed through textwrap.dedent first.
+    :param str module_name: The name of the module.
+    :returns: The designated node from the parse tree, or a list of nodes.
+    :rtype: astroid.bases.NodeNG, or a list of nodes.
+    """
+    def _extract(node):
+        if isinstance(node, nodes.Expr):
+            return node.value
+
+        return node
+
+    requested_lines = []
+    for idx, line in enumerate(code.splitlines()):
+        if line.strip().endswith(_STATEMENT_SELECTOR):
+            requested_lines.append(idx + 1)
+
+    tree = parse(code, module_name=module_name)
+    extracted = []
+    if requested_lines:
+        for line in requested_lines:
+            extracted.append(_find_statement_by_line(tree, line))
+
+    # Modifies the tree.
+    extracted.extend(_extract_expressions(tree))
+
+    if not extracted:
+        extracted.append(tree.body[-1])
+
+    extracted = [_extract(node) for node in extracted]
+    if len(extracted) == 1:
+        return extracted[0]
+    return extracted
