@@ -1,20 +1,9 @@
-# copyright 2003-2015 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
-# contact http://www.logilab.fr/ -- mailto:contact@logilab.fr
-#
-# This file is part of astroid.
-#
-# astroid is free software: you can redistribute it and/or modify it
-# under the terms of the GNU Lesser General Public License as published by the
-# Free Software Foundation, either version 2.1 of the License, or (at your
-# option) any later version.
-#
-# astroid is distributed in the hope that it will be useful, but
-# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License
-# for more details.
-#
-# You should have received a copy of the GNU Lesser General Public License along
-# with astroid. If not, see <http://www.gnu.org/licenses/>.
+# Copyright (c) 2015-2016 Cara Vinson <ceridwenv@gmail.com>
+# Copyright (c) 2015-2016 Claudiu Popa <pcmanticore@gmail.com>
+
+# Licensed under the LGPL: https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html
+# For details: https://github.com/PyCQA/astroid/blob/master/COPYING.LESSER
+
 
 """
 Various helper utilities.
@@ -26,6 +15,7 @@ from astroid import bases
 from astroid import context as contextmod
 from astroid import exceptions
 from astroid import manager
+from astroid import nodes
 from astroid import raw_building
 from astroid import scoped_nodes
 from astroid import util
@@ -83,17 +73,19 @@ def _object_type(node, context=None):
 def object_type(node, context=None):
     """Obtain the type of the given node
 
+    This is used to implement the ``type`` builtin, which means that it's
+    used for inferring type calls, as well as used in a couple of other places
+    in the inference.
     The node will be inferred first, so this function can support all
-    sorts of objects, as long as they support inference. It will try to
-    retrieve the Python type, as returned by the builtin `type`.
+    sorts of objects, as long as they support inference.
     """
 
     try:
         types = set(_object_type(node, context))
     except exceptions.InferenceError:
-        return util.YES
+        return util.Uninferable
     if len(types) > 1 or not types:
-        return util.YES
+        return util.Uninferable
     return list(types)[0]
 
 
@@ -137,7 +129,7 @@ def has_known_bases(klass, context=None):
 
 def _type_check(type1, type2):
     if not all(map(has_known_bases, (type1, type2))):
-        return util.YES
+        raise exceptions._NonDeducibleTypeHierarchy
 
     if not all([type1.newstyle, type2.newstyle]):
         return False
@@ -145,7 +137,7 @@ def _type_check(type1, type2):
         return type1 in type2.mro()[:-1]
     except exceptions.MroError:
         # The MRO is invalid.
-        return util.YES
+        raise exceptions._NonDeducibleTypeHierarchy
 
 
 def is_subtype(type1, type2):
@@ -156,3 +148,26 @@ def is_subtype(type1, type2):
 def is_supertype(type1, type2):
     """Check if *type2* is a supertype of *type1*."""
     return _type_check(type1, type2)
+
+
+def class_instance_as_index(node):
+    """Get the value as an index for the given instance.
+
+    If an instance provides an __index__ method, then it can
+    be used in some scenarios where an integer is expected,
+    for instance when multiplying or subscripting a list.
+    """
+    context = contextmod.InferenceContext()
+    context.callcontext = contextmod.CallContext(args=[node])
+
+    try:
+        for inferred in node.igetattr('__index__', context=context):
+            if not isinstance(inferred, bases.BoundMethod):
+                continue
+
+            for result in inferred.infer_call_result(node, context=context):
+                if (isinstance(result, nodes.Const)
+                        and isinstance(result.value, int)):
+                    return result
+    except exceptions.InferenceError:
+        pass
