@@ -1,20 +1,9 @@
-# copyright 2003-2015 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
-# contact http://www.logilab.fr/ -- mailto:contact@logilab.fr
-#
-# This file is part of astroid.
-#
-# astroid is free software: you can redistribute it and/or modify it
-# under the terms of the GNU Lesser General Public License as published by the
-# Free Software Foundation, either version 2.1 of the License, or (at your
-# option) any later version.
-#
-# astroid is distributed in the hope that it will be useful, but
-# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License
-# for more details.
-#
-# You should have received a copy of the GNU Lesser General Public License along
-# with astroid. If not, see <http://www.gnu.org/licenses/>.
+# Copyright (c) 2015-2016 Cara Vinson <ceridwenv@gmail.com>
+# Copyright (c) 2015-2016 Claudiu Popa <pcmanticore@gmail.com>
+
+# Licensed under the LGPL: https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html
+# For details: https://github.com/PyCQA/astroid/blob/master/COPYING.LESSER
+
 
 """
 Inference objects are a way to represent composite AST nodes,
@@ -23,24 +12,6 @@ original AST tree. For instance, inferring the following frozenset use,
 leads to an inferred FrozenSet:
 
     Call(func=Name('frozenset'), args=Tuple(...))
-
-
-This distinction might help for understanding better where to use
-an AST node and where to use an inference object:
-
-    * if the AST node can be used on its own, e.g. List, Tuple, etc,
-      then the AST can be used.
-      They have a syntactic display and can be written on its own,
-      e.g. "[]" is a valid list. That's why the inference of their
-      respective builtins (list, tuple, set etc) returns an AST node
-      when inferring.
-
-    * if the AST node is part of syntax and can't be used
-      on its own. This is the case for slices for instance, e.g "[2:3:4]".
-      In this case, trying to write "2:3:4" on its own will fail.
-      The respective builtin (slice) returns a custom object when inferring
-      and not an AST node per se.
-
 """
 
 import six
@@ -55,6 +26,7 @@ from astroid import util
 
 
 BUILTINS = six.moves.builtins.__name__
+objectmodel = util.lazy_import('interpreter.objectmodel')
 
 
 class FrozenSet(node_classes._BaseContainer):
@@ -67,38 +39,12 @@ class FrozenSet(node_classes._BaseContainer):
         yield self
 
     @decorators.cachedproperty
-    def _proxied(self):
+    def _proxied(self): # pylint: disable=method-hidden
         builtins = MANAGER.astroid_cache[BUILTINS]
         return builtins.getattr('frozenset')[0]
 
 
-class Slice(node_classes.Slice):
-    """Custom object for representing slices internally."""
-
-    @decorators.cachedproperty
-    def _proxied(self):
-        builtins = MANAGER.astroid_cache[BUILTINS]
-        return builtins.getattr('slice')[0]
-
-    def pytype(self):
-        return '%s.slice' % BUILTINS
-
-    def igetattr(self, attrname, context=None):
-        if attrname == 'start':
-            yield self.lower
-        elif attrname == 'stop':
-            yield self.upper
-        elif attrname == 'step':
-            yield self.step
-        else:
-            for value in self.getattr(attrname, context=context):
-                yield value
-
-    def getattr(self, attrname, context=None):
-        return self._proxied.getattr(attrname, context)
-
-
-class Super(bases.NodeNG):
+class Super(node_classes.NodeNG):
     """Proxy class over a super call.
 
     This class offers almost the same behaviour as Python's super,
@@ -110,19 +56,16 @@ class Super(bases.NodeNG):
     *self_class* is the class where the super call is, while
     *scope* is the function where the super call is.
     """
+    # pylint: disable=unnecessary-lambda
+    special_attributes = util.lazy_descriptor(lambda: objectmodel.SuperModel())
 
+    # pylint: disable=super-init-not-called
     def __init__(self, mro_pointer, mro_type, self_class, scope):
         self.type = mro_type
         self.mro_pointer = mro_pointer
         self._class_based = False
         self._self_class = self_class
         self._scope = scope
-        self._model = {
-            '__thisclass__': self.mro_pointer,
-            '__self_class__': self._self_class,
-            '__self__': self.type,
-            '__class__': self._proxied,
-        }
 
     def _infer(self, context=None):
         yield self
@@ -130,8 +73,9 @@ class Super(bases.NodeNG):
     def super_mro(self):
         """Get the MRO which will be used to lookup attributes in this super."""
         if not isinstance(self.mro_pointer, scoped_nodes.ClassDef):
-            raise exceptions.SuperArgumentTypeError(
-                "The first super argument must be type.")
+            raise exceptions.SuperError(
+                "The first argument to super must be a subtype of "
+                "type, not {mro_pointer}.", super_=self)
 
         if isinstance(self.type, scoped_nodes.ClassDef):
             # `super(type, type)`, most likely in a class method.
@@ -140,18 +84,20 @@ class Super(bases.NodeNG):
         else:
             mro_type = getattr(self.type, '_proxied', None)
             if not isinstance(mro_type, (bases.Instance, scoped_nodes.ClassDef)):
-                raise exceptions.SuperArgumentTypeError(
-                    "super(type, obj): obj must be an "
-                    "instance or subtype of type")
+                raise exceptions.SuperError(
+                    "The second argument to super must be an "
+                    "instance or subtype of type, not {type}.",
+                    super_=self)
 
         if not mro_type.newstyle:
-            raise exceptions.SuperError("Unable to call super on old-style classes.")
+            raise exceptions.SuperError("Unable to call super on old-style classes.", super_=self)
 
         mro = mro_type.mro()
         if self.mro_pointer not in mro:
-            raise exceptions.SuperArgumentTypeError(
-                "super(type, obj): obj must be an "
-                "instance or subtype of type")
+            raise exceptions.SuperError(
+                "The second argument to super must be an "
+                "instance or subtype of type, not {type}.",
+                super_=self)
 
         index = mro.index(self.mro_pointer)
         return mro[index + 1:]
@@ -175,18 +121,25 @@ class Super(bases.NodeNG):
     def igetattr(self, name, context=None):
         """Retrieve the inferred values of the given attribute name."""
 
-        local_name = self._model.get(name)
-        if local_name:
-            yield local_name
+        if name in self.special_attributes:
+            yield self.special_attributes.lookup(name)
             return
 
         try:
             mro = self.super_mro()
-        except (exceptions.MroError, exceptions.SuperError) as exc:
-            # Don't let invalid MROs or invalid super calls
-            # to leak out as is from this function.
-            util.reraise(exceptions.NotFoundError(*exc.args))
-
+        # Don't let invalid MROs or invalid super calls
+        # leak out as is from this function.
+        except exceptions.SuperError as exc:
+            util.reraise(exceptions.AttributeInferenceError(
+                ('Lookup for {name} on {target!r} because super call {super!r} '
+                 'is invalid.'),
+                target=self, attribute=name, context=context, super_=exc.super_))
+        except exceptions.MroError as exc:
+            util.reraise(exceptions.AttributeInferenceError(
+                ('Lookup for {name} on {target!r} failed because {cls!r} has an '
+                 'invalid MRO.'),
+                target=self, attribute=name, context=context, mros=exc.mros,
+                cls=exc.cls))
         found = False
         for cls in mro:
             if name not in cls.locals:
@@ -206,11 +159,62 @@ class Super(bases.NodeNG):
                     yield inferred
                 elif self._class_based or inferred.type == 'staticmethod':
                     yield inferred
+                elif bases._is_property(inferred):
+                    # TODO: support other descriptors as well.
+                    for value in inferred.infer_call_result(self, context):
+                        yield value
                 else:
                     yield bases.BoundMethod(inferred, cls)
 
         if not found:
-            raise exceptions.NotFoundError(name)
+            raise exceptions.AttributeInferenceError(target=self,
+                                                     attribute=name,
+                                                     context=context)
 
     def getattr(self, name, context=None):
         return list(self.igetattr(name, context=context))
+
+
+class ExceptionInstance(bases.Instance):
+    """Class for instances of exceptions
+
+    It has special treatment for some of the exceptions's attributes,
+    which are transformed at runtime into certain concrete objects, such as
+    the case of .args.
+    """
+
+    # pylint: disable=unnecessary-lambda
+    special_attributes = util.lazy_descriptor(lambda: objectmodel.ExceptionInstanceModel())
+
+
+class DictInstance(bases.Instance):
+    """Special kind of instances for dictionaries
+
+    This instance knows the underlying object model of the dictionaries, which means
+    that methods such as .values or .items can be properly inferred.
+    """
+
+    # pylint: disable=unnecessary-lambda
+    special_attributes = util.lazy_descriptor(lambda: objectmodel.DictModel())
+
+
+# Custom objects tailored for dictionaries, which are used to
+# disambiguate between the types of Python 2 dict's method returns
+# and Python 3 (where they return set like objects).
+class DictItems(bases.Proxy):
+    __str__ = node_classes.NodeNG.__str__
+    __repr__ = node_classes.NodeNG.__repr__
+
+
+class DictKeys(bases.Proxy):
+    __str__ = node_classes.NodeNG.__str__
+    __repr__ = node_classes.NodeNG.__repr__
+
+
+class DictValues(bases.Proxy):
+    __str__ = node_classes.NodeNG.__str__
+    __repr__ = node_classes.NodeNG.__repr__
+
+# TODO: Hack to solve the circular import problem between node_classes and objects
+# This is not needed in 2.0, which has a cleaner design overall
+node_classes.Dict.__bases__ = (node_classes.NodeNG, DictInstance)
